/*
 * This file is part of ViaBackwards - https://github.com/ViaVersion/ViaBackwards
 * Copyright (C) 2016-2023 ViaVersion and contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.viaversion.viabackwards.protocol.protocol1_13_2to1_14.packets;

import com.viaversion.viabackwards.ViaBackwards;
import com.viaversion.viabackwards.api.rewriters.SoundRewriter;
import com.viaversion.viabackwards.protocol.protocol1_13_2to1_14.Protocol1_13_2To1_14;
import com.viaversion.viabackwards.protocol.protocol1_13_2to1_14.storage.EntityPositionStorage1_14;
import com.viaversion.viaversion.api.data.entity.StoredEntityData;
import com.viaversion.viaversion.api.protocol.packet.PacketWrapper;
import com.viaversion.viaversion.api.rewriter.RewriterBase;
import com.viaversion.viaversion.api.type.Type;
import com.viaversion.viaversion.protocols.protocol1_13to1_12_2.ClientboundPackets1_13;
import com.viaversion.viaversion.protocols.protocol1_14to1_13_2.ClientboundPackets1_14;

public class SoundPackets1_14 extends RewriterBase<Protocol1_13_2To1_14> {

    public SoundPackets1_14(Protocol1_13_2To1_14 protocol) {
        super(protocol);
    }

    @Override
    protected void registerPackets() {
        SoundRewriter<ClientboundPackets1_14> soundRewriter = new SoundRewriter<>(protocol);
        soundRewriter.registerSound(ClientboundPackets1_14.SOUND);
        soundRewriter.registerNamedSound(ClientboundPackets1_14.NAMED_SOUND);
        soundRewriter.registerStopSound(ClientboundPackets1_14.STOP_SOUND);

        // Entity Sound Effect
        protocol.registerClientbound(ClientboundPackets1_14.ENTITY_SOUND, null, wrapper -> {
            wrapper.cancel();

            int soundId = wrapper.read(Type.VAR_INT);
            int newId = protocol.getMappingData().getSoundMappings().getNewId(soundId);
            if (newId == -1) return;

            int category = wrapper.read(Type.VAR_INT);
            int entityId = wrapper.read(Type.VAR_INT);

            StoredEntityData storedEntity = wrapper.user().getEntityTracker(protocol.getClass()).entityData(entityId);
            EntityPositionStorage1_14 entityStorage;
            if (storedEntity == null || (entityStorage = storedEntity.get(EntityPositionStorage1_14.class)) == null) {
                ViaBackwards.getPlatform().getLogger().warning("Untracked entity with id " + entityId);
                return;
            }

            float volume = wrapper.read(Type.FLOAT);
            float pitch = wrapper.read(Type.FLOAT);
            int x = (int) (entityStorage.getX() * 8D);
            int y = (int) (entityStorage.getY() * 8D);
            int z = (int) (entityStorage.getZ() * 8D);

            PacketWrapper soundPacket = wrapper.create(ClientboundPackets1_13.SOUND);
            soundPacket.write(Type.VAR_INT, newId);
            soundPacket.write(Type.VAR_INT, category);
            soundPacket.write(Type.INT, x);
            soundPacket.write(Type.INT, y);
            soundPacket.write(Type.INT, z);
            soundPacket.write(Type.FLOAT, volume);
            soundPacket.write(Type.FLOAT, pitch);
            soundPacket.send(Protocol1_13_2To1_14.class);
        });
    }
}
